EasyPet = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
EasyPet.options = AceLibrary("Dewdrop-2.0")
EasyPet:RegisterDB("EasyPetDB")
EasyPet:RegisterDefaults("profile", {
    Locked = true,
    Scale = .8,
    }
)

-- FIND SPELL NUMBER
local function GetSpellID(spell)
    local i = 1
    while true do
       local spellName, spellRank = GetSpellName(i, BOOKTYPE_PET)
       if not spellName then
          do break end
       end
       
       if (spellName == spell) then return i end
       
       i = i + 1
    end
end

function EasyPet:OnInitialize()
    EasyPetOptionsAnchor = CreateFrame("Frame", "EasyPetOptionsAnchor", UIParent)
    EasyPetOptionsAnchor:SetWidth(50); EasyPetOptionsAnchor:SetHeight(50); EasyPetOptionsAnchor:SetAlpha(0)
    EasyPetOptionsAnchor:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    EasyPetOptionsAnchor:EnableMouse(false); EasyPetOptionsAnchor:Hide()
    
    if not EasyPet:ClassCheck() then return end
    
    self.opt = self.db.profile
end

function EasyPet:OnEnable()
    if not EasyPet:ClassCheck() then return end

    self:RegisterEvent("PET_BAR_UPDATE")
    self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
    self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE")
    self:RegisterEvent("CHAT_MSG_SPELL_BREAK_AURA")
    self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER")
    
    -- OPTIONS MENU
    for varname, val in pairs(EasyPet_Placement.args) do
        if self.opt[varname] then  EasyPet_Placement.args[varname].set(true) end
    end

    EasyPet.options:Register("EasyPetOptionsAnchor",
        'children', function(level, value) EasyPet:PopulateOptions(level, value) end,
        'point', function(parent) return "CENTER", "CENTER" end,
        'dontHook', true
    )

    -- LOAD BUTTONS
    EasyPet:LoadPetButtons()

    -- HIDE THEM ALL
    for i = 1, 8 do
        local button = getglobal("EasyPetButton"..i)
        if button then button:Hide() end
    end

    -- THEN SHOW THE CORRECT ONES
    EasyPet:UpdateButtons()

    -- SHOW OR HIDE THE ANCHOR
    if not EasyPet.db.profile.Locked then
        EasyPetAnchor:EnableMouse(0); EasyPetAnchor:Hide()
    else
        EasyPetAnchor:EnableMouse(1); EasyPetAnchor:Show()
    end
end

function EasyPet:PopulateOptions(level, value)
    if level == 1 then
        EasyPet.options:AddLine('text', "EasyPet", 'isTitle', true)
        EasyPet.options:AddLine('text', "")
        EasyPet.options:AddLine( 'text', EasyPetLocals["Messages"]["Placement"], 'hasArrow', true, 'value', EasyPetLocals["Messages"]["Placement"])
        EasyPet.options:AddLine( 'text', EasyPetLocals["Messages"]["Scale"], 'hasArrow', true, 'value', EasyPetLocals["Messages"]["Scale"])
        EasyPet.options:FeedAceOptionsTable(EasyPet_Drag, 0)
        EasyPet.options:AddLine('text', "")
        EasyPet.options:AddLine( 'text', EasyPetLocals["Messages"]["Read"], 'hasArrow', true, 'value', EasyPetLocals["Messages"]["Read"])
        EasyPet.options:AddLine('text', "")
        EasyPet.options:AddLine('text', EasyPetLocals["Messages"]["Close"], 'func', function() EasyPet.options:Close(1) end)

    elseif level == 2 then
        if value == EasyPetLocals["Messages"]["Placement"] then
            EasyPet.options:FeedAceOptionsTable(EasyPet_Placement, 1)
        elseif value == EasyPetLocals["Messages"]["Scale"] then
            EasyPet.options:FeedAceOptionsTable(EasyPet_Scale, 1)
        elseif value == EasyPetLocals["Messages"]["Read"] then
            EasyPet.options:FeedAceOptionsTable(EasyPet_Notice, 1)
        end
    end
end

EasyPet:RegisterChatCommand("/EasyPet", function() EasyPet.options:Open("EasyPetOptionsAnchor") end)

function EasyPet:LoadPetButtons()
    -- BUTTON ANCHOR
    EasyPetAnchor = CreateFrame("Frame", "EasyPetAnchor", UIParent)
    EasyPetAnchor:SetWidth(35); EasyPetAnchor:SetHeight(35)
    EasyPetAnchor:SetMovable(true); EasyPetAnchor:EnableMouse(true); 
    EasyPetAnchor:SetPoint("CENTER", UIParent, "CENTER", 0, -275)
    
    EasyPetAnchor:SetScript("OnMouseDown", function() EasyPetAnchor:StartMoving() end)
    EasyPetAnchor:SetScript("OnMouseUp", function() EasyPetAnchor:StopMovingOrSizing() end)
    EasyPetAnchor:SetScript("OnDragStop", function() EasyPetAnchor:StopMovingOrSizing() end)
    EasyPetAnchor:SetScript("OnLeave", function() GameTooltip:Hide() end)
    EasyPetAnchor:SetScript("OnEnter", function() GameTooltip:SetOwner(this, "ANCHOR_CURSOR"); GameTooltip:SetText(EasyPetLocals["Messages"]["LeftToDrag"]) end)

    local t = EasyPetAnchor:CreateTexture(nil,"BACKGROUND")
    t:SetTexture("Interface\\Addons\\EasyPet\\anchor.tga")
    t:SetAllPoints(EasyPetAnchor)
    EasyPetAnchor.texture = t

    -- CREATE 8 BUTTONS
    EasyPetButton1 = CreateFrame("Button", "EasyPetButton1", UIParent, "EasyPetButtonTemplate")
    EasyPetButton2 = CreateFrame("Button", "EasyPetButton2", UIParent, "EasyPetButtonTemplate")
    EasyPetButton3 = CreateFrame("Button", "EasyPetButton3", UIParent, "EasyPetButtonTemplate")
    EasyPetButton4 = CreateFrame("Button", "EasyPetButton4", UIParent, "EasyPetButtonTemplate")
    EasyPetButton5 = CreateFrame("Button", "EasyPetButton5", UIParent, "EasyPetButtonTemplate")
    EasyPetButton6 = CreateFrame("Button", "EasyPetButton6", UIParent, "EasyPetButtonTemplate")
    EasyPetButton7 = CreateFrame("Button", "EasyPetButton7", UIParent, "EasyPetButtonTemplate")
    EasyPetButton8 = CreateFrame("Button", "EasyPetButton8", UIParent, "EasyPetButtonTemplate")

    for i = 1, 8 do
        local button = getglobal("EasyPetButton"..i)
        button:SetWidth(60); button:SetHeight(60)
        button:SetMovable(true); button:EnableMouse(true)
        button:SetParent("PetActionBarFrame")
        button:SetScript("OnLeave", function() GameTooltip:Hide() end)
        button:SetScript("OnEnter", function()
            if IsMounted() then return
            elseif UnitIsDeadOrGhost("player") then return
            elseif not IsUsableSpell(EasyPetLocals["Spells"][i]) then return
            end
            GameTooltip:SetOwner(this, "ANCHOR_CURSOR")
            GameTooltip:SetSpell(GetSpellID(EasyPetLocals["Spells"][i]), BOOKTYPE_PET)
            GameTooltip:Show()
        end)
        
        local t = button:CreateTexture(nil,"BACKGROUND")
        t:SetTexture(EasyPetLocals["Icons"][i])
        t:SetAllPoints(button)
        button.texture = t
    end
    
    EasyPetButton1:SetAttribute("type", "spell")
    EasyPetButton1:SetAttribute("unit", "player")
    EasyPetButton1:SetAttribute("spell", EasyPetLocals["Spells"][1])
    EasyPetButton1:SetPoint("CENTER", EasyPetAnchor, "CENTER", -60, -100)
    
    EasyPetButton2:SetAttribute("type", "spell")
    EasyPetButton2:SetAttribute("unit", "pet")
    EasyPetButton2:SetAttribute("spell", EasyPetLocals["Spells"][2])
    EasyPetButton2:SetPoint("CENTER", EasyPetButton1, "RIGHT", 30, -30)
    
    EasyPetButton3:SetAttribute("type", "spell")
    EasyPetButton3:SetAttribute("spell", EasyPetLocals["Spells"][3])
    EasyPetButton3:SetPoint("CENTER", EasyPetButton2, "RIGHT", 30, 30)
    
    EasyPetButton4:SetAttribute("type", "spell")
    EasyPetButton4:SetAttribute("spell", EasyPetLocals["Spells"][4])
    EasyPetButton4:SetPoint("CENTER", EasyPetButton1, "RIGHT", 30, 30)
    
    EasyPetButton5:SetAttribute("type", "spell")
    EasyPetButton5:SetAttribute("spell", EasyPetLocals["Spells"][5])
    EasyPetButton5:SetPoint("CENTER", EasyPetAnchor, "CENTER", 0, -130)
    
    EasyPetButton6:SetAttribute("type", "spell")
    EasyPetButton6:SetAttribute("spell", EasyPetLocals["Spells"][6])
    EasyPetButton6:SetPoint("CENTER", EasyPetAnchor, "CENTER", -30, -130)
    
    EasyPetButton7:SetAttribute("type", "spell")
    EasyPetButton7:SetAttribute("spell", EasyPetLocals["Spells"][7])
    EasyPetButton7:SetPoint("CENTER", EasyPetButton6, "RIGHT", 30, 0)
    
    EasyPetButton8:SetAttribute("type", "spell")
    EasyPetButton8:SetAttribute("spell", EasyPetLocals["Spells"][8])
    EasyPetButton8:SetPoint("CENTER", EasyPetAnchor, "CENTER", 0, -130)
end

-- COOLDOWN UPDATE
function EasyPet:SPELL_UPDATE_COOLDOWN()
    if not UnitAffectingCombat("player") then
        EasyPet:UpdateButtons()
    end
    
    if (GetSpellCooldown(EasyPetLocals["Spells"][1]) ~= nil and GetSpellCooldown(EasyPetLocals["Spells"][1]) > 0) then
        local start, duration, enable = GetSpellCooldown(EasyPetLocals["Spells"][1])
        EasyPetButton1Cooldown:SetCooldown(start, duration)
        EasyPetButton2Cooldown:SetCooldown(start, duration)
        EasyPetButton3Cooldown:SetCooldown(start, duration)
    end
    if (GetSpellCooldown(EasyPetLocals["Spells"][4]) ~= nil and GetSpellCooldown(EasyPetLocals["Spells"][4]) > 0) then
        local start, duration, enable = GetSpellCooldown(EasyPetLocals["Spells"][4])
        EasyPetButton4Cooldown:SetCooldown(start, duration)
    end
    if (GetSpellCooldown(EasyPetLocals["Spells"][5]) ~= nil and GetSpellCooldown(EasyPetLocals["Spells"][5]) > 0) then
        local start, duration, enable = GetSpellCooldown(EasyPetLocals["Spells"][5]);
        EasyPetButton5Cooldown:SetCooldown(start, duration)
    end
    if (GetSpellCooldown(EasyPetLocals["Spells"][8]) ~= nil and GetSpellCooldown(EasyPetLocals["Spells"][8]) > 0) then
        local start, duration, enable = GetSpellCooldown(EasyPetLocals["Spells"][8])
        EasyPetButton8Cooldown:SetCooldown(start, duration)
    end
end

-- UPDATE BUTTONS
function EasyPet:UpdateButtons()
    if not UnitCreatureFamily("pet") then return end
    local pet = UnitCreatureFamily("pet")
    
    if (pet == EasyPetLocals["Pets"][2]) then
        EasyPetButton1:Show()
        EasyPetButton2:Show()
        EasyPetButton3:Show()
        EasyPetButton4:Show()
        
        for i = 5, 8 do
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    elseif (pet == EasyPetLocals["Pets"][3]) then
        EasyPetButton5:Show()
        
        for i = 1, 8 do
            if i == 5 then break end
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    elseif (pet == EasyPetLocals["Pets"][4]) then
        EasyPetButton6:Show()
        EasyPetButton7:Show()
        
        for i = 1, 8 do
            if i == 6 then break end
            if i == 7 then break end
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    elseif (pet == EasyPetLocals["Pets"][5]) then
        EasyPetButton8:Show()
        
        for i = 1, 7 do
            local button = getglobal("EasyPetButton"..i)
            if button then button:Hide() end
        end
    end

    for i = 1, 8 do
        local button = getglobal("EasyPetButton"..i)
        button:SetScale(EasyPet.db.profile.Scale)
    end
end

function EasyPet:PET_BAR_UPDATE()
    if not UnitAffectingCombat("player") then
        EasyPet:UpdateButtons()
    end
end

function EasyPet:PLAYER_REGEN_ENABLED()
    if not UnitAffectingCombat("player") then
        EasyPet:UpdateButtons()
    end
end

-- CHECK CLASS
function EasyPet:ClassCheck()
    local _, class = UnitClass("player")
    if class == "WARLOCK" then return true end
end

-- SEDUCTION TIMER
function EasyPet:SeductionTimer(time)
    if not UnitCreatureFamily("pet") then return end
    local pet = UnitCreatureFamily("pet")
    
    if (pet == EasyPetLocals["Pets"][3]) and string.find(string.lower(arg1), string.lower(EasyPetLocals["Spells"][5])) then
        EasyPetButton5Cooldown:SetCooldown(GetTime(), time)
    end
end

function EasyPet:CHAT_MSG_SPELL_AURA_GONE_OTHER(arg1)
    EasyPet:SeductionTimer(0)
end

function EasyPet:CHAT_MSG_SPELL_BREAK_AURA(arg1)
    EasyPet:SeductionTimer(0)
end

function EasyPet:CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE(arg1)
    EasyPet:SeductionTimer(12)
end

function EasyPet:CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE(arg1)
    EasyPet:SeductionTimer(15)
end